/*-----------------------------------------------------------------
  All code is provided for EDUCATIONAL purposes ONLY. There is no
  warranty and no support. Licensed under the MIT license - see
  http://www.qsl.net/ve3lny/MIT%20License.html

  interpolate.c

  This module contains the code to convert the ADC sample to
  tenths of ohms.

  The maximum input occurs when the test probes are shorted,
  which corresponds to 0 ohms. We want to remember this value
  and store it in eeprom so it can be set up each time the unit
  is powered on. This is done when the zero button is pressed.
  The 'zero' value will be highly dependent on the hardware in
  the esr circuit itself, but should hopefully be consistant
  once set.

  Otherwise, diminishing input values represent increasing
  resistance, and not on a linear scale either.
-----------------------------------------------------------------*/
#include <string.h>
#include <avr/io.h>
#include "common.h"

#define NR_ENTS		35		// number of table entries

extern volatile uint16_t uiZeroValueMV;

/*---------------------------------------------------------------------
  interpolate(...)

  Converts the voltage to resistance, using linear interpolation.

  Linear interpolation takes two data points, say (Xa,Ya) and (Xb,Yb),
  and the interpolant (X,Y) is given by:

           (X - Xa) (Yb - Ya)
  Y = Ya + ------------------
               (Xb - Xa)

  In this case X is voltage and Y is resistance.

  Input: Millivolts (0..5000)

  Output: Tenths of Ohms (0..990)
---------------------------------------------------------------------*/
uint16_t interpolate( uint16_t uiMV )
{
  static int16_t iMilliVolts[NR_ENTS] =
  {
	SHORT_MV, 4220, 4190, 4130, 4120, 4060, 3990, 3730, 3490, 3250,
	    3110, 2890, 2740, 2320, 1818, 1582, 1396, 1130,  841,  653,
		 555,  470,  421,  365,  310,  258,  223,  195,  177,  146,
		 124,  104,   81,   67,    0
  };
  static int16_t iOhmsTenths[NR_ENTS] =
  {
		   0,    2,    3,    5,    6,    8,   10,   20,   30,   40,  
		  50,   60,   70,  100,  150,  180,  210,  260,  330,  400,
		 444,  480,  510,  550,  600,  640,  680,  720,  740,  790,
		 830,  870,  940,  980,  999 
  };

  uint8_t	ii;
  uint16_t	usResult;
  int16_t	sTerm1, sTerm2, sTerm3;

  // Check for boundary values
  if ( uiMV >= uiZeroValueMV ) return 0;
  if ( !uiMV ) return 999;

  iMilliVolts[0] = uiZeroValueMV;		// always use current value

  for ( ii=0; ii<NR_ENTS; ++ii )
  {
    if ( uiMV >= (uint16_t) iMilliVolts[ii] )
      break;
  }

  if ( uiMV == (uint16_t) iMilliVolts[ii] )
    return (uint16_t) iOhmsTenths[ii];		// no interpolation necessary

  sTerm1 = (int16_t) uiMV - iMilliVolts[ii-1];
  sTerm2 = iOhmsTenths[ii] - iOhmsTenths[ii-1];
  sTerm3 = iMilliVolts[ii] - iMilliVolts[ii-1];
  usResult = (uint16_t) ( iOhmsTenths[ii-1] + ( ( sTerm1 * sTerm2 ) / sTerm3 ) );

  return usResult;
}

