/*-----------------------------------------------------------------
  All code is provided for EDUCATIONAL purposes ONLY. There is no
  warranty and no support. Licensed under the MIT license - see
  http://www.qsl.net/ve3lny/MIT%20License.html

  led.c

  This modules supports a 2 digit 7 segment common anode display.
-----------------------------------------------------------------*/
#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include "common.h"
#include "led.h"

extern volatile uint16_t	uiTenthsOhms;		// tenths of ohms
extern volatile char		cDisplay[2];		// digits to display
extern volatile char 		cDecimal[2];		// decimal point state

/*---------------------------------------------------------------------
  ledSelectDigit(...)

  Selects a digit of the led display. There are 2 digits selected by
  ports that drive the common pin of each display.

  Input: a single numeric digit, range 0..1. 0 drives the high order
         digit, 1 drives the low order digit. To turn off all digits,
         pass NO_DIGIT.
---------------------------------------------------------------------*/
void ledSelectDigit( char cDigit )
{

  switch ( cDigit )
  {
    case 0:
      clear_bit( LED_DIGIT_1_PORT, LED_DIGIT_1 );
      break;
    case 1:
      clear_bit( LED_DIGIT_2_PORT, LED_DIGIT_2 );
      break;
    case NO_DIGIT:
      set_bit( LED_DIGIT_1_PORT, LED_DIGIT_1 );
      set_bit( LED_DIGIT_2_PORT, LED_DIGIT_2 );
	  set_bit( LED_DECIMAL_PORT, LED_DECIMAL );
  }
}

/*---------------------------------------------------------------------
  ledOutputNumber(...)

  Outputs a number to the led display. Ports PA4..PA7 are connected
  to the A-B-C-D inputs of a 74LS47 which decodes the number and
  activates the corresponding segments of the bank of LEDs. 

  Input: 1. a single numeric digit, range 0..9 (ascii or binary, only the
            low-order nibble is processed.)
		 2. The decimal point state (on/off)
---------------------------------------------------------------------*/
void ledOutputNumber( char cNumber, char cDecimal )
{
  // Output the digit as BCD on PAx..PAy

  if ( test_bit( cNumber, 0 ) )
    set_bit( LED_ENCODER_PORT, LED_ENCODER_A );
  else
    clear_bit( LED_ENCODER_PORT, LED_ENCODER_A );

  if ( test_bit( cNumber, 1 ) )
    set_bit( LED_ENCODER_PORT, LED_ENCODER_B );
  else
    clear_bit( LED_ENCODER_PORT, LED_ENCODER_B );

  if ( test_bit( cNumber, 2 ) )
    set_bit( LED_ENCODER_PORT, LED_ENCODER_C );
  else
    clear_bit( LED_ENCODER_PORT, LED_ENCODER_C );

  if ( test_bit( cNumber, 3 ) )
    set_bit( LED_ENCODER_PORT, LED_ENCODER_D );
  else
    clear_bit( LED_ENCODER_PORT, LED_ENCODER_D );

  if ( cDecimal )
	clear_bit( LED_DECIMAL_PORT, LED_DECIMAL );
  else
	set_bit( LED_DECIMAL_PORT, LED_DECIMAL );
}

/*---------------------------------------------------------------------
  format_counter()

  Format our data for the 2 digit led display. When the value is less
  than 10 ohms (100 tenths), x.x (ohms) is displayed. When the value is
  10 or greater, xx ohms is displayed and the fraction is dropped.

  Input: global variable uiTenthsOhms

  Output: global variables cDisplay, cDecimal
---------------------------------------------------------------------*/
void format_counter()
{
  char		buffer[8];
  uint8_t	nn;

  if ( uiTenthsOhms < 0 || uiTenthsOhms > 999 )
  {
    // Value out of range: show 99
    cDisplay[0] = cDisplay[1] = '9';
	cDecimal[0] = cDecimal[1] = 0;
    return;
  }

  if ( !uiTenthsOhms )
  {
    // Value zero: display .0
    cDisplay[0] = 0x0f;
    cDisplay[1] = '0';	
	cDecimal[0] = 1;
	cDecimal[1] = 0;
    return;
  }

  utoa( uiTenthsOhms, buffer, 10 );
  nn = (uint8_t) strlen( buffer );

  if ( uiTenthsOhms > 99 )
  {
	// Display whole ohms, no decimal
	cDecimal[0] = cDecimal[1] = 0;
    cDisplay[0] = buffer[0]; 
    cDisplay[1] = buffer[1];
	return;
  }

  // Display tenths of ohms, 1 decimal
  cDecimal[0] = 1;
  cDecimal[1] = 0;

  if ( uiTenthsOhms > 9 )
  {
    // Show 2 digits
    cDisplay[0] = buffer[0]; 
    cDisplay[1] = buffer[1];
  }
  else
  {
	cDisplay[0] = 0x0f;
    cDisplay[1] = buffer[0];
  }
}
