/*-----------------------------------------------------------------
  All code is provided for EDUCATIONAL purposes ONLY. There is no
  warranty and no support. Licensed under the MIT license - see
  http://www.qsl.net/ve3lny/MIT%20License.html

  main.c

  ESR meter digital display:

  Provides a 2 digit digital LED display for an ESR meter (see
  http://ludens.cl/Electron/esr/esr.html). The output of the
  ESR measuring circuit is input to ADC0, converted to ohms and
  displayed on the 2 digit LED display. Note that the ohms
  displayed is inversely proportional to the input signal, and 
  non-linear as well, so a look-up table and interpolation is
  used to derive the reading.

  Processor: ATtiny461 or ATtiny861
      Clock: Internal, 4.0 MHz

  jb20110204 - Program written.
-----------------------------------------------------------------*/
#include <avr/io.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <util/delay_basic.h>
#include "common.h"
#include "led.h"

#define INIT_CTR0L   	0xf7	// 65536-15625 = 49911d or C2F7h
#define INIT_CTR0H   	0xc2
#define INIT_CTR1	 	6		// 256 - 250

#define EVENT_ZEROBTN_PRESSED	0
#define EVENT_ADC_COMPLETE		1

// Global variables

volatile char 		cDisplay[2] = { 0, 0 };	// digits to display
volatile char 		cDecimal[2] = { 0, 0 };	// decimal point state

volatile uint16_t	uiTenthsOhms=0;			// display data
volatile uint16_t	uiZeroValueMV=SHORT_MV;

static volatile uint8_t		ucEventFlags=0;		// event flags
static volatile uint16_t	uiAdc=0;			// last adc sample

// Internal Functions

static void system_init(void);

/*-----------------------------------------------------------------
  main()
-----------------------------------------------------------------*/
int main(void)
{
  uint16_t uiMV;

  system_init();			// initialise processor

  uiZeroValueMV = eeprom_read_word( 0 );
  if ( uiZeroValueMV > SHORT_MV )
  {
    uiZeroValueMV = SHORT_MV;
	eeprom_write_word( 0, uiZeroValueMV );
  }

  format_counter();			// initial update the display

  while( 1 )
  {
	if ( test_bit( ucEventFlags, EVENT_ADC_COMPLETE ) )
	{
	  // millivolts = ( ADC * 5000 ) / 1024

	  uiMV = (uint16_t) ( ( (uint32_t) uiAdc * 5000L ) / 1024L );

	  uiTenthsOhms = interpolate( uiMV );

	  format_counter();			// update the display

	  clear_bit( ucEventFlags, EVENT_ADC_COMPLETE );
	}

	if ( test_bit( ucEventFlags, EVENT_ZEROBTN_PRESSED ) )
	{
	  // Remember the MV corresponding to the zero button press
	  uiZeroValueMV = (uint16_t) ( ( (uint32_t) uiAdc * 5000L ) / 1024L );
	  eeprom_write_word( 0, uiZeroValueMV );

	  clear_bit( ucEventFlags, EVENT_ZEROBTN_PRESSED );
	}

	_delay_loop_1( 133 );		// 100 us delay

  }					// end while(1)
}

/*-----------------------------------------------------------------
  Timer 0 overflow interrupt handler, for timing ADC conversions

  Since the clock is running at 4.0 mHz, the counter increments
  every 250.0 ns. Using a prescaler of 64 gives a cycle time of
  16us. So if we count to 15,625 we get 250ms interrupts.
-----------------------------------------------------------------*/
ISR(TIMER0_OVF_vect)
{
  // Initiate an ADC conversion.
  set_bit( ADCSRA, ADSC );				// initiate conversion

  TCNT0L = INIT_CTR0L;					// reinitialize counter
  TCNT0H = INIT_CTR0H;
}

/*-----------------------------------------------------------------
  Timer 1 overflow interrupt handler - tests for button presses
  and switches LED digits

  Since the clock is running at 4.0 mHz, the counter increments
  every 250 ns. Using a prescaler of 128 gives us 32us. If we count
  250 times we get an interrupt every 8 ms.

  To achieve an interval of approximately 8 ms for selecting
  the next LED digit. By sequencing each digit fast enough, the
  LED digits both appear to be on.

  Test the button every interrupt and signal an event when
  the button is pressed for enough consecutive intervals.
-----------------------------------------------------------------*/
ISR(TIMER1_OVF_vect)
{
  static volatile uint8_t ucIndex=0;
  static volatile uint8_t ucZeroSw=0, ucZeroSwLatch=0;

  // Select and display the next LED digit

  ledSelectDigit( NO_DIGIT );               // turn all digits off

  ledOutputNumber( cDisplay[ucIndex], cDecimal[ucIndex] );	// output the number

  ledSelectDigit( ucIndex );                // select the digit

  ++ucIndex;
  if ( ucIndex == 2 ) ucIndex = 0;		// cycle the digit index

  // Read current input states

  if ( test_bit( PINB, PB6 ) )			// button open
  {
    ucZeroSw = 0;
	ucZeroSwLatch = 0;
  }
  else									// button closed
	++ucZeroSw;

  // Create button press events

  if ( ucZeroSw > 5 )
  {
	if ( !ucZeroSwLatch )
	{
	  set_bit( ucEventFlags, EVENT_ZEROBTN_PRESSED );
	  ucZeroSwLatch = 1;		// to avoid multiple events
	}
  }

  TCNT1 = INIT_CTR1;			// reinitialize the timer
}

/*-----------------------------------------------------------------
  ISR( ADC_vect ) - ADC conversion complete interrupt handler
-----------------------------------------------------------------*/
ISR( ADC_vect )
{
  // ADCL must be read before ADCH.
  uiAdc = (uint16_t) ADCL | ( (uint16_t) ADCH << 8 );
  set_bit( ucEventFlags, EVENT_ADC_COMPLETE );
}

/*-----------------------------------------------------------------
  system_init()

  PORTB: PB0 (input)  MOSI (ISP)
         PB1 (input)  MISO (ISP)
         PB2 (input)  SCK (ISP)
         PB3 (output) display decimal point
         PB4 (input)  not used
         PB5 (output) DIG1 select 
         PB6 (input)  Zero Button
         PB7 (input)  !Reset (ISP)

  PORTA: PA0 (input)  A/D from ESR meter
         PA1 (input)  not used
         PA2 (output) DIG2 select  
         PA3 (input)  A/D Aref  
         PA4 (output) LED BCD "A"
         PA5 (output) LED BCD "B"
         PA6 (output) LED BCD "C" 
         PA7 (output) LED BCD "D"

-----------------------------------------------------------------*/
static void system_init()
{
  // The fuses are set to provide an internal clock, running at
  // 8.0 MHz with a prescaler of 8. Change the prescaler from
  // 8 to 2. The procedure requires two steps as per the device
  // specifications.
  CLKPR = 0b10000000;       // set clock prescaler change enable
  CLKPR = 0b00000001;       // set prescaler to 2

  DDRB  = 0b00101000;       // PortB: 0=input, 1 = output
  PORTB = 0b01000000;       // initialise PortB, pullup 6

  DDRA  = 0b11110100;       // PortA: 0=input, 1 = output
  PORTA = 0b00000000;       // initialise PortA

  // Initialize Timer0. Timer0 is used to produce 100ms ticks
  // for various timing purposes.
  TCCR0A = 0b10000000;       // normal 16-bit counter mode
  TCCR0B = 0b00000011;       // prescaler = 64 (turns timer0 on)
  TCNT0L = INIT_CTR0L;
  TCNT0H = INIT_CTR0H;
  set_bit( TIMSK, TOIE0 );	// enable timer/counter0 overflow interrupt

  // Initialize Timer1. Timer1 is used to debounce button presses
  // and switch LED digits every 8 ms.
  TCCR1A = 0b00000000;		// normal operation
  TCCR1B = 0b00001000;		// prescaler = 128 (turns timer1 on)
  TCNT1 = INIT_CTR1;
  set_bit( TIMSK, TOIE1 );	// enable timer/counter1 overflow interrupt

  // Set up the ADC on port ADC0, using Vcc as reference
  ADMUX  = 0b00000000;		// Vref=Vcc, right adjusted, Input=ADC0
  ADCSRA = 0b10001101;		// enable ADC; prescaler=32; enable interrupt
  ADCSRB = 0b00000000;		// REFS2=0

  DIDR0  = 0b00001001;		// disable digital buffer: ADC0 AREF
  DIDR1  = 0b00000000;

  sei();                    // enable global interrupts
}

